library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity divider is
	generic
	(
		bits	:	integer := MWIDTH
	);
	port
	(
		--inputs
		X	:	in	std_logic_vector(2*bits-1 downto 0);
		Y	:	in	std_logic_vector(2*bits-1 downto 0);
		CLK	:	in	std_logic;
		--outputs
		RESULT	:	out	std_logic_vector(2*bits+1 downto 0)
	);	
end entity;


architecture divider_arch of divider is

component reciprocal_MWIDTH is
	port
	(
		addr: IN std_logic_VECTOR(bits-1 downto 0);
	        clk: IN std_logic;
	        dout: OUT std_logic_VECTOR(2*bits+3 downto 0)
	);
end component;

component multiplier1_MWIDTH IS
	port (
		clk: IN std_logic;
		a: IN std_logic_VECTOR(2*bits-1 downto 0);
		b: IN std_logic_VECTOR(2*bits-1 downto 0);
		o: OUT std_logic_VECTOR(4*bits-1 downto 0));
	END component;

component multiplier2_MWIDTH IS
	port (
		clk: IN std_logic;
		a: IN std_logic_VECTOR(2*bits+3 downto 0);
		b: IN std_logic_VECTOR(2*bits+3 downto 0);
		o: OUT std_logic_VECTOR(4*bits+7 downto 0));
END component;


type rArray is array(PIPELINE downto 0) of std_logic_vector(2*bits+3 downto 0);
	
--======================================================--
--                      SIGNALS                         --
--======================================================--
signal  x_1 : std_logic_vector(2*bits-1 downto 0);

signal	yh : std_logic_vector(2*bits-1 downto 0);
signal	yl : std_logic_vector(2*bits-1 downto 0);

signal	y_sub : std_logic_vector(2*bits-1 downto 0);
signal	y_sub_1 : std_logic_vector(2*bits-1 downto 0);

signal  yhl : std_logic_vector(bits-1 downto 0);
signal  yhl_1 : std_logic_vector(bits-1 downto 0);
	
signal  reciprocals : rArray;

signal  mul11 : std_logic_vector(4*bits-1 downto 0);
signal  mul1s : std_logic_vector(2*bits+3 downto 0);
	
signal  mul2 : std_logic_vector(4*bits+7 downto 0);
signal  mul2s : std_logic_vector(2*bits+2 downto 0);
	
signal  zero : std_logic_vector(2*bits-1 downto 0);
signal  round1 : std_logic_vector(2*bits+1 downto 0);
signal  res1 : std_logic_vector(2*bits+1 downto 0);
signal  round2 : std_logic_vector(2*bits+1 downto 0);
signal  res2 : std_logic_vector(2*bits+1 downto 0);
signal  res : std_logic_vector(2*bits+1 downto 0);
	
	
begin
----------------------------------------------------------
--               Component Instantiation                --
----------------------------------------------------------
bram: reciprocal_MWIDTH
	port map
	(
		--inputs
		addr	=>	yhl_1,
		CLK	=>	clk,
		--outputs
		dout	=>	reciprocals(0)
	);	
		
mul_1 : multiplier1_MWIDTH
	port map
	(
		clk => clk,
		a => x_1,
		b => y_sub_1,
		o => mul11
	);
	
mul_2 : multiplier2_MWIDTH
	port map
	(
		clk => clk,
		a => mul1s,
		b => reciprocals(PIPELINE),
		o => mul2
	);

	
----------------------------------------------------------
--              Asynchronous Assignments                --
----------------------------------------------------------
	yhl <= y(2*bits-2 downto bits-1);
		
	yh(2*bits-1 downto bits-1) <= y(2*bits-1 downto bits-1);
	yh(bits-2 downto 0) <= (others=>'0');
	
	yl(2*bits-1 downto bits-1) <= (others=>'0');
	yl(bits-2 downto 0) <= y(bits-2 downto 0);
	
	y_sub <= yh-yl;
	
	zero <= (others=>'0');
	round1 <= zero & '0' & mul2s(0);
	res1 <= mul2s(2*bits+2 downto 1) + round1;
	round2 <= zero & res2(0) & '0';
	res <= res2(2*bits+1 downto 1)&'0' + round2;

	regs : for i in 1 to PIPELINE generate
        	reciprocals(i) <= reciprocals(i-1) when rising_edge(clk);
        end generate;

	
----------------------------------------------------------
--               Synchronous Assignments                --
----------------------------------------------------------
synchro: process (CLK)
begin
	if(rising_edge(CLK)) then
	        x_1 <= X;
	        y_sub_1 <= y_sub;
	        yhl_1 <= yhl;
	        mul1s <= mul11(4*bits-1 downto 2*bits-4);	
		mul2s <= mul2(4*bits+5 downto 2*bits+3);
		res2  <= res1;
		RESULT 	<= res;

	end if;--CLK
end process;--synchro

end divider_arch; --end of architecture

