library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_signed.all; -- signed, 2's complement number


entity sq is
	generic
	(
	   bits			:	integer := KWIDTH
	);
	port
	(
		--inputs
	 	odd_in		:	in      std_logic;
		Y_in		:	in	std_logic_vector(4*bits-1 downto 0); 
		CLK		:	in	std_logic;
		--outputs
		res		:	out	std_logic_vector(4*bits-1 downto 0) 
	);	
end sq;


architecture arch of sq is

	component r_KWIDTH is
		port (
		Addr: IN std_logic_VECTOR(bits-1 downto 0);
		clk : in std_logic;
		dout: OUT std_logic_VECTOR(bits+1 downto 0)
	);
	end component;


	component m_mul2_KWIDTH IS
		port (
		addr: IN std_logic_VECTOR(bits-1 downto 0);
		clk: IN std_logic;
		dout: OUT std_logic_VECTOR(4*bits+1 downto 0));
	END component;


	component m_KWIDTH IS
		port (
		addr: IN std_logic_VECTOR(bits-1 downto 0);
		clk: IN std_logic;
		dout: OUT std_logic_VECTOR(4*bits-2 downto 0));
	END component;


	component multiplier_yr_KWIDTH IS
		port (
		clk: in std_logic;
       		a: IN std_logic_VECTOR(bits+2 downto 0);
		b: IN std_logic_VECTOR(4*bits+1 downto 0);
		o: OUT std_logic_VECTOR(5*bits+4 downto 0));
   	 END component;

	
	component multiplier_s_KWIDTH IS
		port (
		clk: in std_logic;
        	a: IN std_logic_VECTOR(bits-1 downto 0);
		b: IN std_logic_VECTOR(bits-1 downto 0);
		o: OUT std_logic_VECTOR(2*bits-1 downto 0));
	END component;
	

	component multiplier_s_lut0_2 IS
		port (
		a: IN std_logic_VECTOR(bits-1 downto 0);
		b: IN std_logic_VECTOR(bits-1 downto 0);
		o: OUT std_logic_VECTOR(2*bits-1 downto 0));
	END component;
	

	component multiplier_m_KWIDTH IS
		port (
		clk: IN std_logic;
		a: IN std_logic_VECTOR(2*bits-1 downto 0);
		b: IN std_logic_VECTOR(bits-1 downto 0);
		o: OUT std_logic_VECTOR(3*bits-1 downto 0));
	END component;
	

	component multiplier_l_KWIDTH IS
		port (
		clk: IN std_logic;
		a: IN std_logic_VECTOR(4*bits+1 downto 0);
		b: IN std_logic_VECTOR(4*bits+1 downto 0);
		o: OUT std_logic_VECTOR(8*bits+3 downto 0));
	END component;


	component parameterized_or_gate is
		generic
		(
			bits	:	integer
		);
		port
		(
			A	:	in	std_logic_vector(bits-1 downto 0);
			O	:	out	std_logic
		);
	end component;
	

type mArray is array(PIPELINE downto 0) of std_logic_vector(4*bits+1 downto 0);


--======================================================--
--                      SIGNALS                         --
--======================================================--
signal	y : std_logic_vector(4*bits-1 downto 0);
signal	yk : std_logic_vector(bits-1 downto 0);
signal  y_ext : std_logic_vector(4*bits+1 downto 0);
signal  r : std_logic_vector(bits+1 downto 0);
signal  r_ext : std_logic_vector(bits+2 downto 0);

signal  a_mul : std_logic_vector(5*bits+4 downto 0);
signal  a_mul_r : std_logic_vector(4*bits+1 downto 0);
signal  a_mul_r_short : std_logic_vector(1 downto 0);
signal  one : std_logic_vector(1 downto 0);
signal  a_mul_r_subone_short : std_logic_vector(1 downto 0);

signal  a_r : std_logic_vector(4*bits+1 downto 0); 
signal  a_r_neg : std_logic_vector(4*bits+1 downto 0); 
	
signal  a_half : std_logic_vector(4*bits+1 downto 0);
signal  a2_r : std_logic_vector(bits-1 downto 0);
signal  a2_neg : std_logic_vector(bits-1 downto 0);
signal  a2_pos : std_logic_vector(bits-1 downto 0);
signal  a2_pos_r : std_logic_vector(bits-1 downto 0);
signal  a3_r : std_logic_vector(bits-1 downto 0);
signal  a3_neg : std_logic_vector(bits-1 downto 0);
signal  a3_pos : std_logic_vector(bits-1 downto 0);
	
signal  mul_a2_a2_short_r : std_logic_vector(2*bits-1 downto 0);
signal  zero_a2_a2 : std_logic_vector(2*bits+4 downto 0);	
signal  mul_a2_a2_ext : std_logic_vector(4*bits+3 downto 0);
signal  mul_a2_a2 : std_logic_vector(4*bits+1 downto 0);
	
signal  mul_a2_a3_short_r : std_logic_vector(2*bits-1 downto 0);
signal  zero_a2_a3 : std_logic_vector(3*bits+3 downto 0);	
signal  mul_a2_a3_ext : std_logic_vector(4*bits+3 downto 0);
signal  mul_a2_a3 : std_logic_vector(4*bits+1 downto 0);
signal  mul_a2_a3_r : std_logic_vector(4*bits+1 downto 0);
	
signal  mul_a2_a2_a2_short_r : std_logic_vector(3*bits-1 downto 0);
signal  mul_a2_a2_a2_short_r_neg : std_logic_vector(3*bits-1 downto 0);
signal  zero_mul_a2_a2_a2_short_r : std_logic;
signal  zero_a2_a2_a2 : std_logic_vector(3*bits+5 downto 0);	
signal  one_a2_a2_a2 : std_logic_vector(3*bits+5 downto 0);	
signal  mul_a2_a2_a2_ext : std_logic_vector(4*bits+5 downto 0);        
signal  mul_a2_a2_a2 : std_logic_vector(4*bits+1 downto 0);
signal  mul_a2_a2_a2_r : std_logic_vector(4*bits+1 downto 0);        

signal  sum_two_short : std_logic_vector(1 downto 0);
signal  sum_two : std_logic_vector(4*bits+1 downto 0);
signal  sum_two_r : std_logic_vector(4*bits+1 downto 0);

signal  sum_three : std_logic_vector(4*bits+1 downto 0);
signal  sum_three_r : std_logic_vector(4*bits+1 downto 0);

signal  sum_four : std_logic_vector(4*bits+1 downto 0);
signal  sum_four_r : std_logic_vector(4*bits+1 downto 0);
signal  sum_four_r_r : std_logic_vector(4*bits+1 downto 0);

signal  a_sign : std_logic; 
signal  a_sign_1 : std_logic; 
signal  a_sign_2 : std_logic; 

signal  b : std_logic_vector(4*bits+1 downto 0); 
signal  b_r : std_logic_vector(4*bits+1 downto 0); 

signal  m : std_logic_vector(4*bits-2 downto 0); 
signal  m_r_1 : std_logic_vector(4*bits+1 downto 0); 
signal  m_mul2 : std_logic_vector(4*bits+1 downto 0); 
signal  m_mul2_r_1 : std_logic_vector(4*bits+1 downto 0); 
signal  m_corr : mArray;

signal  mul_m_b : std_logic_vector(8*bits+3 downto 0);
 
signal  odd : std_logic;
signal  odd_1 : std_logic;
signal  odd_2 : std_logic;
	
	
begin
----------------------------------------------------------
--               Component Instantiation                --
----------------------------------------------------------
r_table: r_KWIDTH
	port map
	(
		addr	=>	yk,
		clk	=>	clk,
		dout	=>	r
	);	
	
	
m_table: m_KWIDTH
	port map
	(
		addr	=>	yk,
		clk	=>	clk,
		dout	=>	m
	);	
	

m_mul2_table: m_mul2_KWIDTH
	port map
	(
		addr	=>	yk,
		clk	=>	clk,
		dout	=>	m_mul2
	);	

	
multiplier_y_r : multiplier_yr_KWIDTH
	port map
	(
		clk	=>	clk,
        	a	=>	r_ext,
        	b	=>	y_ext,
        	o	=>	a_mul
	);     


 G_mul_s_1: if (bits>=3) generate       
	multiplier_a2_a2 : multiplier_s_KWIDTH
		port map
		(
			clk => clk,
            		a => a2_pos,
            		b => a2_pos,
            		o => mul_a2_a2_short_r
       		);     
       
	multiplier_a2_a3 : multiplier_s_KWIDTH
		port map
		(
			clk => clk,
            		a => a2_pos,
            		b => a3_pos,
            		o => mul_a2_a3_short_r
       );     
 end generate G_mul_s_1;


 G_mul_s_2: if (bits=2) generate       
	multiplier_a2_a2 : multiplier_s_lut0_2
		port map
		(
	        	a => a2_pos,
            		b => a2_pos,
            		o => mul_a2_a2_short_r
       		);     
 end generate G_mul_s_2;


 G_mul_m_1: if (bits>=5) generate
 	a2_pos_r <= a2_pos when rising_edge(clk);
    
	multiplier_a2_a2_a2 : multiplier_m_KWIDTH
	port map
		(
			clk => clk,
                	a => mul_a2_a2_short_r,
  	        	b => a2_pos_r,
  	        	o => mul_a2_a2_a2_short_r
  		);     
 end generate G_mul_m_1;    


multiplier_m_b : multiplier_l_KWIDTH
	port map
	(
	        clk => clk,
        	a => m_corr(PIPELINE),
        	b => b_r,
        	o => mul_m_b
       );     

       
all_zero_a2_a2_a2_short: parameterized_or_gate
	generic map
	(
		bits	=>	3*bits
	)
	port map
	(
		A	=>	mul_a2_a2_a2_short_r,
		O	=>	zero_mul_a2_a2_a2_short_r
	);


	
----------------------------------------------------------
--              Asynchronous Assignments                --
----------------------------------------------------------

-- reduction
yk <= y(4*bits-1 downto 3*bits);
r_ext <= '0' & r;
	
a_mul_r <= a_mul(5*bits+2 downto bits+1);
a_mul_r_short <= a_mul_r(4*bits+1 downto 4*bits);             
one <= "01";
a_mul_r_subone_short <= a_mul_r_short - one;
a_r <= a_mul_r_subone_short & a_mul_r(4*bits-1 downto 0);
    
   
-- estimation
-- a_half, sum_two 
G_a_1: if (bits>=2) generate
	a_half <= a_r(4*bits+1) & a_r(4*bits+1 downto 1) when rising_edge(clk);
        a_sign <= a_r(4*bits+1) when rising_edge(clk);
        a_r_neg <= -(a_r) when rising_edge(clk);
end generate G_a_1;

G_a_2: if (bits=1) generate
      a_half <= a_r(4*bits+1) & a_r(4*bits+1 downto 1);
      a_sign <= a_r(4*bits+1);
      a_r_neg <= -(a_r);
end generate G_a_2;    
    
sum_two_short <= "01" + a_half(4*bits+1 downto 4*bits);
sum_two <= sum_two_short & a_half(4*bits-1 downto 0);

    
-- a2xa2, sum_three
G_sum_three_1: if (bits>=3) generate
  	a2_r <= a_r(3*bits-1 downto 2*bits) when rising_edge(clk);
  	a2_neg <= a_r_neg(3*bits-1 downto 2*bits);
  	a2_pos <= a2_r when a_sign = '0' else    
               a2_neg;

	zero_a2_a2 <= (others=>'0');
	mul_a2_a2_ext <= zero_a2_a2 & mul_a2_a2_short_r(2*bits-1 downto 1);
	mul_a2_a2 <= mul_a2_a2_ext(4*bits+3 downto 2);

        sum_two_r <= sum_two when rising_edge(clk);   	
    	sum_three <= sum_two_r - mul_a2_a2;	
end generate G_sum_three_1;    	
    
G_sum_three_2: if (bits=2) generate
    	a2_r <= a_r(3*bits-1 downto 2*bits) when rising_edge(clk);
    	a2_neg <= a_r_neg(3*bits-1 downto 2*bits);
    	a2_pos <= a2_r when a_sign = '0' else    
                 a2_neg;

      	zero_a2_a2 <= (others=>'0');
	mul_a2_a2_ext <= zero_a2_a2 & mul_a2_a2_short_r(2*bits-1 downto 1);
 	mul_a2_a2 <= mul_a2_a2_ext(4*bits+3 downto 2);

  	sum_three <= sum_two - mul_a2_a2;	
end generate G_sum_three_2;    	    
    
G_sum_three_3: if (bits=1) generate
        sum_three <= sum_two;	
end generate G_sum_three_3;    	


-- a2xa3, sum_four
G_sum_four_1: if (bits>=3) generate
	a3_r <= a_r(2*bits-1 downto bits) when rising_edge(clk);
	a3_neg <= a_r_neg(2*bits-1 downto bits);
        a3_pos <= a3_r when a_sign = '0' else
	          a3_neg;          
    
       	zero_a2_a3 <= (others=>'0');
	mul_a2_a3_ext <= zero_a2_a3 & mul_a2_a3_short_r(2*bits-1 downto bits);
	mul_a2_a3 <= mul_a2_a3_ext(4*bits+3 downto 2);
	mul_a2_a3_r <= mul_a2_a3 when rising_edge(clk);

        sum_three_r <= sum_three when rising_edge(clk);
    	sum_four <= sum_three_r - mul_a2_a3_r;   	
end generate G_sum_four_1;
    
G_sum_four_2: if (bits<3) generate
        sum_four <= sum_three;   	
end generate G_sum_four_2;    	

    
-- a2^3, b
a_sign_1 <= a_sign when rising_edge(clk);
a_sign_2 <= a_sign_1 when rising_edge(clk);

mul_a2_a2_a2_short_r_neg <= -(mul_a2_a2_a2_short_r);
zero_a2_a2_a2 <= (others=>'0');
one_a2_a2_a2 <= (others=>'1');
mul_a2_a2_a2_ext <= zero_a2_a2_a2 & mul_a2_a2_a2_short_r(3*bits-1 downto 2*bits) when a_sign_2='0' or zero_mul_a2_a2_a2_short_r='0' else
                    one_a2_a2_a2 & mul_a2_a2_a2_short_r_neg(3*bits-1 downto 2*bits);
mul_a2_a2_a2 <= mul_a2_a2_a2_ext(4*bits+5 downto 4);                    
mul_a2_a2_a2_r <= mul_a2_a2_a2 when rising_edge(clk);
	
sum_four_r <= sum_four when rising_edge(clk);

G_b_0: if (bits>=9) generate
	sum_four_r_r <= sum_four_r when rising_edge(clk);
   	b <= sum_four_r_r + mul_a2_a2_a2_r;
end generate G_b_0;    	
    
G_b_1: if (bits>=5 and bits<9) generate
    	b <= sum_four_r + mul_a2_a2_a2_r;
end generate G_b_1;    	
    
G_b_2: if (bits<5) generate
	b <= sum_four;
end generate G_b_2;    	

    
-- post processing
m_corr(0) <= m_mul2_r_1 when odd_2='1' else
             m_r_1;


regs : for i in 1 to PIPELINE generate
	m_corr(i) <= m_corr(i-1) when rising_edge(clk);
end generate;
	
y <= y_in;
odd <= odd_in;

	
----------------------------------------------------------
--               Synchronous Assignments                --
----------------------------------------------------------
synchro: process (CLK)
begin
	if(rising_edge(CLK)) then
		y_ext <= "01" & y;	        	
        	b_r <= b;

        	m_r_1 <= "010" & m;
        	m_mul2_r_1 <= m_mul2;

   		odd_1 <= odd;
   		odd_2 <= odd_1;
   			 
        	res <= mul_m_b(8*bits-1 downto 4*bits);
	end if;
end process;

end arch; 